@@ -1,6 +1,8 @@
+from __future__ import print_function
+
 import sys
 import argparse
-from Bio import SeqIO
+from Bio.SeqIO.FastaIO import SimpleFastaParser
 
 parser = argparse.ArgumentParser(description='Partition a genome into a set of overlapping segments')
 parser.add_argument('--segment-length', type=int, default=50000)
@@ -10,7 +12,9 @@ if len(extra) != 1:
     sys.stderr.write("Error: a genome file is expected\n")
 filename = extra[0]
 
-recs = [ (rec.name, len(rec.seq)) for rec in SeqIO.parse(open(filename), "fasta")]
+with open(filename) as handle:
+    recs = [(title.split(None, 1)[0], len(seq))
+            for title, seq in SimpleFastaParser(handle)]
 
 SEGMENT_LENGTH = args.segment_length
 OVERLAP_LENGTH = args.overlap_length
