@@ -92,7 +92,7 @@ SquiggleRead::SquiggleRead(const std::string& name, const ReadDB& read_db, const
         //fprintf(stderr, "type: %s\n", experiment_type.c_str());
 
         // Try to detect whether this read is DNA or RNA
-        this->nucleotide_type = experiment_type == "rna" ? SRNT_RNA : SRNT_DNA;
+        this->nucleotide_type = experiment_type == "rna" || experiment_type == "internal_rna" ? SRNT_RNA : SRNT_DNA;
 
         // Did this read come from nanopolish extract?
         bool is_event_read = is_extract_read_name(this->read_name);
@@ -321,7 +321,7 @@ void SquiggleRead::load_from_raw(hid_t hdf5_file, const uint32_t flags)
     assert(rt.n > 0);
     assert(et.n > 0);
 
-    // 
+    //
     this->scalings[strand_idx] = estimate_scalings_using_mom(this->read_sequence,
                                                              *this->base_model[strand_idx],
                                                              et);
@@ -336,6 +336,7 @@ void SquiggleRead::load_from_raw(hid_t hdf5_file, const uint32_t flags)
     }
 
     if(flags & SRF_LOAD_RAW_SAMPLES) {
+        this->sample_start_time = 0;
         this->samples.resize(rt.n);
         for(size_t i = 0; i < this->samples.size(); ++i) {
             assert(rt.start + i < rt.n);
@@ -343,7 +344,7 @@ void SquiggleRead::load_from_raw(hid_t hdf5_file, const uint32_t flags)
         }
     }
 
-    // If sequencing RNA, reverse the events to be 3'->5'
+    // If sequencing RNA, reverse the events to be 5'->3'
     if(this->nucleotide_type == SRNT_RNA) {
         std::reverse(this->events[strand_idx].begin(), this->events[strand_idx].end());
     }
@@ -1085,20 +1086,16 @@ std::vector<EventAlignment> SquiggleRead::get_eventalignment_for_1d_basecalls(co
 
 size_t SquiggleRead::get_sample_index_at_time(size_t sample_time) const
 {
-    return sample_time - sample_start_time;
+    return sample_time - this->sample_start_time;
 }
 
 //
 std::vector<float> SquiggleRead::get_scaled_samples_for_event(size_t strand_idx, size_t event_idx) const
 {
-    double event_start_time = this->events[strand_idx][event_idx].start_time;
-    double event_duration = this->events[strand_idx][event_idx].duration;
-
-    size_t start_idx = this->get_sample_index_at_time(event_start_time * this->sample_rate);
-    size_t end_idx = this->get_sample_index_at_time((event_start_time + event_duration) * this->sample_rate);
+    std::pair<size_t, size_t> sample_range = get_event_sample_idx(strand_idx, event_idx);
 
     std::vector<float> out;
-    for(size_t i = start_idx; i < end_idx; ++i) {
+    for(size_t i = sample_range.first; i < sample_range.second; ++i) {
         double curr_sample_time = (this->sample_start_time + i) / this->sample_rate;
         //fprintf(stderr, "event_start: %.5lf sample start: %.5lf curr: %.5lf rate: %.2lf\n", event_start_time, this->sample_start_time / this->sample_rate, curr_sample_time, this->sample_rate);
         double s = this->samples[i];
@@ -1112,6 +1109,18 @@ std::vector<float> SquiggleRead::get_scaled_samples_for_event(size_t strand_idx,
     return out;
 }
 
+// return a pair of value corresponding to the start and end index of a given index on the signal
+std::pair<size_t, size_t> SquiggleRead::get_event_sample_idx(size_t strand_idx, size_t event_idx) const
+{
+    double event_start_time = this->events[strand_idx][event_idx].start_time;
+    double event_duration = this->events[strand_idx][event_idx].duration;
+
+    size_t start_idx = this->get_sample_index_at_time(event_start_time * this->sample_rate);
+    size_t end_idx = this->get_sample_index_at_time((event_start_time + event_duration) * this->sample_rate);
+
+    return std::make_pair(start_idx, end_idx);
+}
+
 void SquiggleRead::detect_pore_type()
 {
     assert(f_p and f_p->is_open());
